<?php if (!defined('BASEPATH')) exit('No direct script access allowed'); 
/**
 * Code Igniter
 *
 * An open source application development framework for PHP 4.3.2 or newer
 *
 * @package		CodeIgniter
 * @author		Rick Ellis
 * @copyright	Copyright (c) 2006, pMachine, Inc.
 * @license		http://www.codeignitor.com/user_guide/license.html
 * @link		http://www.codeigniter.com
 * @since		Version 1.0
 * @filesource
 */

// ------------------------------------------------------------------------



// ------------------------------------------------------------------------

class Imap {

	var $CI;
	var $remote_name;
	var $port;
	var $mailbox;
	var $user;
	var $pass;
	var $mbox;
	var $mbox_list;
	var $stream;
	var $default_mailbox = 'INBOX';
	
	function Imap()
	{	
		$this->CI =& get_instance();
		log_message('debug', "Imap Class Initialized");
	}
	
	function config($config)
	{	
		$this->user = $config['user'];
		$this->pass = $config['pass'];
		$this->server = $config['server'];
		$this->port = $config['port'];
	}

	function connect($user = NULL, $pass = NULL, $server = NULL, $port = NULL)
	{
	
		$this->user		= $this->CI->config->item('imap_user');
		$this->pass		= $this->CI->config->item('imap_pass');
		$this->flags	= $this->CI->config->item('imap_flags');
		$this->port		= $this->CI->config->item('imap_port');
		$this->mailbox	= $this->CI->config->item('imap_mailbox');
		$this->server	= $this->CI->config->item('imap_server');
		
		if($user)
		{
			$this->user = $user;
		}
		
		if($pass)
		{
			$this->pass = $pass;
		}
		
		if($server)
		{
			$this->server = $server;
		}	
		
		if ($port)
		{
			$this->port = $port;
		}

		// Let's start building the stream's string
		$this->mbox		= '{' . gethostbyname($this->server);

		if($this->port)
		{
			$this->mbox .=  ':' . $this->port;
		}
		
		if($this->flags)
		{
			$this->mbox .= $this->flags;
		}

		$this->mbox .= '/user="' . $this->user . '"';
		$this->mbox .= '}';

		if($this->mailbox)
		{
			$this->mbox .= $this->default_mailbox;
		}
		
		$this->stream = @imap_open($this->mbox,$this->user,$this->pass);
		if (FALSE === $this->stream)
		{
			return false;
			return imap_last_error();
		}

		return TRUE;
	}

	function msg_count()
	{
		if($this->stream)
		{
			return imap_num_msg($this->stream);
		}
		
		// Not connected!
		return imap_last_error();
	}
	
	function msg_list($msg_list = array(),$body = false)
	{
		if($this->stream)
		{
			$return = array();
			if(is_array($msg_list) OR count($msg_list) == 0)
			{
				$count = $this->msg_count();
				for($i = 1; $i <= $count; $i++)
				{
					$header = imap_headerinfo($this->stream, $i);
					foreach($header as $id => $value)
					{
						// Simple array
						if(! is_array($value))
						{
							$return[$header->Msgno][$id] = $value;
						}
						else
						{
							foreach($value as $newid => $array_value)
							{
								foreach($value[0] as $key => $aValue)
								{
									$return[$header->Msgno][$id][$key] = $aValue;
								}
							}
						}
						
						// Let's add the body
						// We only want the text, not the extra crap
						if ($body) $return[$header->Msgno]['body'] = imap_fetchbody($this->stream, $header->Msgno,1);
					}
				}
			}
			// We want to search a specific array of messages
			else
			{
				foreach($msg_list as $i)
				{
					$header = imap_headerinfo($this->stream, $i);
					foreach($header as $id => $value)
					{
						// Simple array
						if(! is_array($value))
						{
							$return[$header->Msgno][$id] = $value;
						}
						else
						{
							foreach($value as $newid => $array_value)
							{
								foreach($value[0] as $key => $aValue)
								{
									$return[$header->Msgno][$id][$key] = $aValue;
								}
							}
						}
						// Let's add the body too!
						$return[$header->Msgno]['body'] = imap_fetchbody($this->stream, $header->Msgno, 0);
					}
				}
			}

			#$return['num_of_msgs'] = count($return);
			return $return;
		}
		
		return imap_last_error();
	}
	
	function search($params)
	{
		if($this->stream)
		{
			if(is_array($params))
			{
				$search_string = '';
				foreach($params as $field => $value)
				{
					if(is_numeric($field))
					{
						// Make sure the value is uppercase
						$search_string .= strtoupper($value) . ' ';	
						
					}
					else
					{
						$search_string .= strtoupper($field) . ' "' . $value . '" ';
					}
				}
				
				// Perform the search
				#echo "'$search_string'";
				return imap_search($this->stream, $search_string);
			}
			else
			{
				return imap_last_error();
			}
		}

		return imap_last_error();
	}
	
	function delete($emails, $delete=FALSE)
	{
		if($this->stream)
		{
			if(is_array($emails))
			{
				// Let's delete multiple emails
				if(count($emails) > 0)
				{
					$delete_string = '';
					$email_error = array();
					foreach($emails as $email)
					{
						if($delete)
						{
							if(! @imap_delete($this->stream, $email))
							{
								$email_error[] = $email;
							}
						}
					}
					if(! $delete)
					{
						// Need to take the last comma out!
						$delete_string = implode(',', $emails);
						echo $delete_string;
						imap_mail_move($this->stream, $delete_string, "Inbox/Trash");
						imap_expunge($this->stream);
					}
					else
					{
						// NONE of the emails were deleted
						if(count($email_error) === count($emails))
						{
							return imap_last_error();
						}
						else
						{
							$return['status'] = FALSE;
							$return['not_deleted'] = $email_error;
							return $return;
						}
					}
				}
				else
				{
					return imap_last_error();
				}
			}
			else
			{
				if(! $delete)
				{
					return @imap_mail_move($this->stream, $emails, "INBOX/trash");
				}
				else
				{
					// We only want to delete one email
					if(imap_delete($this->stream, $emails))
					{
						return TRUE;	// Success
					}
					else
					{
						return imap_last_error();	// Failed
					}
				}
			}
		}

		// Not connected
		return imap_last_error();
	}

	function switch_mailbox($mailbox = '')
	{
		if($this->stream)
		{
			$this->mbox = '{' . $this->server;

			if($this->port)
			{
				$this->mbox .=  ':' . $this->port;
			}
			
			if($this->flags)
			{
				$this->mbox .= $this->flags;
			}

			$this->mbox .= ':143/imap/user="' . $this->user . '"';
			$this->mbox .= '}';
			$this->mbox .= $this->default_mailbox;
			
			if($mailbox)
			{
				$this->mbox .= '.' . $mailbox;
			}

			return @imap_reopen($this->stream, $this->mbox);
		}
		
		// Not connected
		return imap_last_error();
	}

	function current_mailbox()
	{
		if($this->stream)
		{
			$info = imap_mailboxmsginfo($this->stream);
			if($info)
			{
				return $info->Mailbox;
			}
			else
			{
				// There was an error
				return imap_last_error();
			}
		}
		
		// Not connected
		return imap_last_error();
	}

	function mailbox_info($type='obj')
	{
		if($this->stream)
		{
			$info = imap_mailboxmsginfo($this->stream);
			if($info)
			{
				if($type == 'array')
				{
					foreach($info as $field => $value)
					{
						$info_array[$field] = $value;
					}
					return $info_array;
				}
				else
				{
					return $info;
				}
			}
			else
			{
				// There was an error
				return imap_last_error();
			}
		}
		
		// Not connected
		return imap_last_error();
	}
	
	function mailboxes()
	{
		if ($this->stream){
			$mailboxes = imap_getmailboxes($this->stream, '{'.$this->server.'}', "*");
		
			foreach ($mailboxes as $mbox){
				$name = str_replace('{'.$this->server.'}','',$mbox->name);
				if (strpos($name,$mbox->delimiter)){
					list($inbox, $mbox_name) = explode($mbox->delimiter, $name);
					$this->mbox_list[] = $mbox_name;
				} else {
					$this->mbox_list[] = 'Inbox';
				}
			}
		
			return $this->mbox_list;
		} else {
			return false;
		}
	}
	
	function get_mime_type(&$structure) { 
	    $primary_mime_type = array("TEXT", "MULTIPART", "MESSAGE", "APPLICATION", "AUDIO", "IMAGE", "VIDEO", "OTHER"); 
	    if($structure->subtype) { 
	         return $primary_mime_type[(int) $structure->type] . '/' . $structure->subtype; 
	     } 
	     return "TEXT/PLAIN"; 
	} 
	
	function get_part($msg_number, $mime_type, $structure = false, $part_number = false) { 
		$this->connect();
	    if (!$structure) { 
	         $structure = imap_fetchstructure($this->stream, $msg_number); 
	    } 
	    print_r($structure);
	    if($structure) { 
			if ($mime_type == $this->get_mime_type($structure)){ 
				if(!$part_number) { 
					$part_number = "1"; 
				} 

				$text = imap_fetchbody($this->stream, $msg_number, $part_number); 

				if($structure->encoding == 3) { 
					return imap_base64($text); 
				} else if ($structure->encoding == 4) { 
					return imap_qprint($text); 
				} else { 
					return $text; 
				}
			} 
			if ($structure->type == 1) { /* multipart */ 
				while (list($index, $sub_structure) = each($structure->parts)) { 
					if ($part_number) { 
						$prefix = $part_number . '.'; 
					} 
					$data = $this->get_part($this->stream, $msg_number, $mime_type, $sub_structure, $prefix . ($index + 1)); 
					if ($data) { 
						return $data; 
					} 
				} 
			} 
		} 
		return false; 
	} 

	
	function close()
	{
		if($this->stream)
		{
			return @imap_close($this->stream);
		}
		
		return imap_last_error();
	}
}

?>