<?php

class CartAPI extends Controller {
	

	function CartAPI(){
		parent::Controller();
		
		$this->load->model('system');
		

		$domain = $_SERVER['REMOTE_ADDR'];
		$api_key = $this->input->post('api_key');
	

		$this->failed = false;


		$allowed_domains = explode(',',str_replace(' ','',$this->system->config('online_sales_allowed_domains')));
		if (in_array($domain, $allowed_domains)){
			if (strtoupper($api_key) != strtoupper($this->system->config('online_sales_api_key'))){
				$return->status = 'Failed';
				$return->reason = 'Invalid API key ' . $api_key . '.';
				
				echo json_encode($return);
				$this->failed = true;
			}	
		} else {
			$return->status = 'Failed';
			$return->reason = 'Access denied from domain ' . $domain . '.';

			echo json_encode($return);
			$this->failed = true;
		}

		

	}
	
	function Order_by_warehouse($output, $args){
		if ($this->failed) return;
		$query = $this->db->query("SELECT * FROM `warehouses`");
		
		if ($query->num_rows() > 1) $data->enabled = true;
		else $data->enabled = false;
		
		$row = $query->row();
		$data->warehouse_id  = $row->id;
			
		$data->module = 'Warehouse';
		if ($output == 'json') echo json_encode($data);
	}
	
	
	
	function Get_online_min_purchase($output, $args){
		if ($this->failed) return;
		$data->min_purchase = $this->system->config('online_sales_min_purchase');	
		$data->currency = $this->system->config('currency');
		if ($output == 'json') echo json_encode($data);
	}
	
	function Get_currency($output, $args){
		if ($this->failed) return;
		$data->currency = $this->system->config('currency');
		if ($output == 'json') echo json_encode($data);
	}
	
	function Get_categories($output, $args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT `category` FROM `inventory` GROUP BY `category` ORDER BY `category` ASC");
		foreach ($query->result() as $item){
			$categories[] = $item->category;
		}
		if ($output == 'json') echo json_encode($categories);
	}
	
	function Get_countries($output, $args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT * FROM `countries`");
		foreach ($query->result() as $country){
			$countries[$country->name] = $country->name;
		}
		if ($output == 'json') echo json_encode($countries);		
		
	}
	
	function Get_subcategories($output, $args){
		if ($this->failed) return;
		
		$category = $this->input->post('cat');
		if ($category){
			$query = $this->db->query("SELECT `subcategory` FROM `inventory` WHERE `category` = ? GROUP BY `subcategory` ORDER BY `subcategory` ASC", array($category));
		} else {
			$query = $this->db->query("SELECT `subcategory` FROM `inventory` GROUP BY `subcategory` ORDER BY `subcategory` ASC");		
		}
		
		foreach ($query->result() as $item){
			$subcategories[] = $item->subcategory;
		}
		
		if ($output == 'json') echo json_encode($subcategories);	
	}
	
	function Get_tags($output, $args){
		if ($this->failed) return;
	
		$query = $this->db->query("SELECT * FROM `inventory` WHERE `tags` LIKE '%#%'");
		foreach ($query->result() as $item){
			if (strpos($item->tags,',')){
				foreach (explode(',',$item->tags) as $tag){
					if ($tag[0] == '#'){
						$tags_array[strtolower(str_replace('#','', $tag))] = str_replace('#','', $tag);
					}
				}
			} else {
				if ($item->tags[0] == '#'){
					$tag = str_replace('#','',$item->tags);
					$tags_array[strtolower($tag)] = $tag;
				}
			}
		}
		
		if ($output == 'json') echo json_encode($tags_array);
		
	}

		
	
	function Get_items($output, $args){
		if ($this->failed) return;
		
		$filter[] = "`linked` IS NULL";
		
		if ($cat = $this->input->post('cat')) $filter[] = "`category` = " . $this->db->escape($cat);
		if ($subcat = $this->input->post('subcat')) $filter[] = "`subcategory` = " . $this->db->escape($subcat);

		if ($term = $this->input->post('term')){
			$filter[] = "(`name` LIKE " . $this->db->escape('%'.$term.'%') . " OR `description` LIKE " . $this->db->escape('%'.$term.'%') . ")";
		}
		
		if ($tag = $this->input->post('tag')){
			$filter[] = "`tags` LIKE " . $this->db->escape('%#'.$tag.'%');
		}
		
		if (!$this->system->config('warehouse_shared_inventory')){
			if ($warehouse_id = $this->input->post('warehouse')) $filter[] = "`warehouse_id` = " . $this->db->escape($warehouse_id);
			else $filter[] = "`warehouse_id` = 1";
		}
		
		$filter[] = " `ready_for_sale` = 1 ";
		
		$warehouse = $this->input->post('warehouse');
		
		$orderby = $this->input->post('orderby');
		$order = $this->input->post('order');
		
		
		if (count($filter)) $where = implode(' AND ', $filter);
		else $where = '1 = 1';
		
		
		
		$limit = $this->system->config('online_sales_item_limit_per_page');
		
		if (!$orderby) $orderby = "`added_on`";
		if (!$order) $order = "ASC";
		
		
		
		if (!$limit) $limit = 3;
		$page = $this->input->post('page') ? $this->input->post('page') : 1;
		$offset_limit = ($page - 1) * $limit;
		
		
		$rules = json_decode($this->system->config('price_rules'));
		
		$query = $this->db->query("SELECT COUNT(*) AS `total` FROM `inventory` WHERE $where");
		$row = $query->row();
		$total = $row->total;
		
		$query = $this->db->query("SELECT * FROM `inventory` WHERE $where ORDER BY $orderby $order LIMIT $offset_limit, $limit");
		foreach ($query->result() as $item){
		
			$query = $this->db->query("SELECT `id`, `name`, `primary_image` FROM `inventory` WHERE `linked` = ?", array($item->sku));
		
			$item->variants = $query->result();
			
			$item->rules = $rules;
			
			$item->currency = $this->system->config('currency');

			$item->warehouse = $warehouse;
			$item->qty = $this->get_item_quantity('return', array('warehouse'=>$warehouse, 'item_id'=>$item->id));
			
			$query = $this->db->query("SELECT * FROM `sales_discount_item_rules` WHERE `item_id` = ? AND `date_from` <= ? AND `date_to` >= ?", array($item->id, date('Y-m-d'),date('Y-m-d')));
			$row = $query->row();
			
			$item->discount_price 					= $row->discount_price;
			$item->discount_percentage 				= $row->discount_percentage;
			$item->discount_applies_to 				= $row->applies_to;
			$item->discount_performance_based 		= $row->is_performance_based;
			$item->discount_performance_duration 	= $row->performance_duration;
			$item->discount_performance_limit 		= $row->performance_limit;



			$items[] = $item;			
		}
		
		
		$result->items = $items;
		$result->limit = $limit;
		$result->total = $total;
		$result->extra = $extra . 'aa';
		
		
		if ($output == 'json') echo json_encode($result);
	
	}
	
	
	function Get_discount_rules($output, $args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT * FROM `sales_discount_item_rules` WHERE `date_from` <= ? AND `date_to` >= ?", array(date('Y-m-d'),date('Y-m-d')));
		
		if ($output == 'json') echo json_encode($query->result());
	}
	
	function Get_item_by_id($output, $args){
		if ($this->failed) return;
		
		$id = $this->input->post('id');
		
		if (is_array($args)){
			foreach ($args as $key => $val){
				${$key} = $val;
			}
		}
		
		$query = $this->db->query("SELECT * FROM `inventory` WHERE `id` = ? ", array($id));
		$item = $query->row();
		
		if ($item->links){
			$query = $this->db->query("SELECT * FROM `inventory` WHERE `linked` = ?", array($item->sku));
			$item->variants = $query->result();
		}
		
		$rules = json_decode($this->system->config('price_rules'));
		
		$item->rules = $rules;
		$item->currency = $this->system->config('currency');
		
		$query = $this->db->query("SELECT * FROM `sales_discount_item_rules` WHERE `item_id` = ? AND `date_from` <= ? AND `date_to` >= ?", array($item->id, date('Y-m-d'),date('Y-m-d')));
		
		if ($query->num_rows()){
			$row = $query->row();
			
			$item->discount_price 					= $row->discount_price;
			$item->discount_percentage 				= $row->discount_percentage;
			$item->discount_applies_to 				= $row->applies_to;
			$item->discount_performance_based 		= $row->is_performance_based;
			$item->discount_performance_duration 	= $row->performance_duration;
			$item->discount_performance_limit 		= $row->performance_limit;
		}
		
		if ($warehouse = $this->input->post('warehouse')){
			$item->warehouse = $warehouse;
			$item->qty = $this->get_item_quantity('return', array('warehouse'=>$warehouse, 'item_id'=>$item->id));
		}

		
		if ($output == 'json') echo json_encode($item);
		elseif ($output == 'return') return $item;
		
	}
	
	function Verify_contacts_by_field($output,$args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `email` = ?", array($this->input->post('email')));
		if ($query->num_rows()){

			$data->data = 'email_false';
			if ($output == 'json') echo json_encode($data);

			return;
		}
		
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `ic` = ?", array($this->input->post('ic')));
		if ($query->num_rows()){

			$data->data = 'ic_false';
			if ($output == 'json') echo json_encode($data);
			return;
		}
		
		
		$data->data = 'ok';
		if ($output == 'json') echo json_encode($data);
		
		
		
	}
	
	function Verify_customer($output, $args){
		if ($this->failed) return;
		
		

		$code = $this->input->post('code');
		$id = $this->input->post('id');		
		
				
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `id` = ?", array($id));
		$customer = $query->row();
		
		if (strpos($customer->tags, ',')){
			$tags = explode(',',$customer->tags);
			foreach ($tags as $tag){
				if ($tag == 'Unverified'){
					$tag = 'Verified';
				}
				$new_tags[] = $tag;
			}
			$tags = implode(',', $new_tags);
		} else {
			$tags = str_replace('Unverified','Verified',$customer->tags);
		}
		
		$this->db->update('contacts', array('tags' => $tags), "`id` = '{$id}'");

		$data->code = $code;
		$data->id = $id;
		$data->customer = $customer;
		

		
		
		if (strtoupper($code) == strtoupper(md5($customer->id.$customer->email.$customer->contact_person))){
			$data->data = 'ok';
		} else {
			$data->data = 'false';
		}

		if ($output == 'json') echo json_encode($data);		
	}	
	
	function Register_customer($output, $args){
		if ($this->failed) return;
	
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `email` = ?", array($this->input->post('email')));
		if ($query->num_rows()){

			$data->data = 'false';
			if ($output == 'json') echo json_encode($data);

		} else {
			
			$ic = ereg_replace("[^A-Za-z0-9\s]", "", $this->input->post('ic') );		
			$introducer_ic = ereg_replace("[^A-Za-z0-9\s]", "", $this->input->post('introducer_ic'));	
			
			$query = $this->db->insert('contacts', array(
				'is_customer'		=> 1,
				'contact_person'	=> $name = $this->input->post('name'),
				'contact_number'	=> $this->input->post('mobile'),
				'address1'			=> $this->input->post('address1'),
				'address2'			=> $this->input->post('address2'),
				'city'				=> $this->input->post('city'),
				'state'				=> $this->input->post('state'),
				'zipcode'			=> $this->input->post('zipcode'),
				'country'			=> $this->input->post('country'),
				'email'				=> $email = $this->input->post('email'),
				'general_email'		=> $email,
				'ic'				=> $ic,
				'introducer_ic'		=> $introducer_ic,				
				'is_member'			=> 1,
				'password'			=> $this->input->post('password'),
				'added_on'			=> date('Y-m-d H:i:s'),
				'added_by'			=> '@Plexcart',
				'tags'				=> 'Online Customer,Unverified'
			));

			$id = $query->insert_id;	
							
			if(strcmp($this->input->post('country'),'Malaysia') != 0){
	
				$sql = $this->db->update_string('contacts',array(
					'ic'	=> $id
				),"`id` = '$id'");
			}
			
			$link = $this->input->post('link') . 'verify/' . $id . '/' . md5($id.$email.$name);
			
			$message = $this->system->config('online_sales_email_new_registration');
			$message = str_replace('{name}', $name, $message);
			$message = str_replace('{email}', $email, $message);
			$message = str_replace('{link}', $link, $message);
			
			
			$config['protocol'] = 'smtp';
			$config['mailpath'] = '/usr/sbin/sendmail';
			$config['charset'] = 'iso-8859-1';
			$config['wordwrap'] = TRUE;
			$config['mailtype'] = 'html';
		
			$config['smtp_host'] = $this->system->config('online_sales_responder_hostname');
			$config['smtp_user'] = $this->system->config('online_sales_responder_email');
			$config['smtp_pass'] = $this->system->config('online_sales_responder_password');
			$config['smtp_port'] = $this->system->config('online_sales_responder_port');

			$this->load->library('email');		
			$this->email->initialize($config);

			$this->email->from($this->system->config('online_sales_responder_email'), $this->system->config('online_sales_responder_name'));
			$this->email->to($email);
			$this->email->bcc($this->system->config('online_sales_responder_txn_email'));
			$this->email->subject($this->system->config('online_sales_email_new_registration_subject'));
			$this->email->message($message);
			$this->email->send();
			
			
			
			$data->data = 'ok';
			if ($output == 'json') echo json_encode($data);
		}
	}
	
	function Test_mail(){

		
		# $config['protocol'] = 'smtp';
		$config['mailpath'] = '/usr/sbin/sendmail';
		$config['charset'] = 'iso-8859-1';
		$config['wordwrap'] = TRUE;
    	$config['mailtype'] = 'html';
		
		$config['smtp_host'] = $this->system->config('online_sales_responder_hostname');
		$config['smtp_user'] = $this->system->config('online_sales_responder_email');
		$config['smtp_pass'] = $this->system->config('online_sales_responder_password');
		$config['smtp_port'] = $this->system->config('online_sales_responder_port');

		$this->load->library('email');		
		$this->email->initialize($config);

		
		$this->email->from($this->system->config('online_sales_responder_email'));
		$this->email->to('shafiq.rizwan@gmail.com');
		$this->email->message('test message');
		$this->email->send();
		
		$data->debug = $this->email->print_debugger();
		echo json_encode($data);
	}
	
	function Update_account($output, $args){
		if ($this->failed) return;
		
		$id = $this->input->post('id');
		
		$this->db->update('contacts', array(
				'company_name'		=> $this->input->post('company'),
				'contact_person'	=> $this->input->post('name'),
				'contact_number'	=> $this->input->post('mobile'),
				'address1'			=> $this->input->post('address1'),
				'address2'			=> $this->input->post('address2'),
				'city'				=> $this->input->post('city'),
				'state'				=> $this->input->post('state'),
				'zipcode'			=> $this->input->post('zipcode'),
				'country'			=> $this->input->post('country'),
				'modified_on'		=> date('Y-m-d H:i:s'),
				'modified_by'		=> '_self',
		),"`id` = '{$id}'");
	
		if ($password = $this->input->post('password')){
			$this->db->update('contacts', array(
				'password'			=> md5($this->input->post('password'))
			), "`id` = '{$id}'");
		}
		
		if ($output == 'json') echo json_encode($data);
	}
	
	function Auth_customer($output, $args){
		if ($this->failed) return;
		
		$username = $this->input->post('username');
		$password = $this->input->post('password');

		$query = $this->db->query("SELECT * FROM `contacts` WHERE `email` = ?", array($username));
		$customer = $query->row();
		
		$tags_exploded = explode(",", $customer->tags);
		
		if ($customer->is_franchise){
			$data->data = 'incorrect';	
			if ($output == 'json') echo json_encode($data);
			
			return;
		}
		
		if (($password == $customer->password) && (in_array('Verified', $tags_exploded))){
		
			$session = $username . time();
			$this->db->query("UPDATE `contacts` SET `current_session` = ? WHERE `email` = ?", array($session, $username));
			
			$data->data = $session;	
		
			if ($output == 'json') echo json_encode($data);
		} else {
		
			$data->data = 'incorrect';	
			if ($output == 'json') echo json_encode($data);
			
		}
		
	}
	
	function Get_customer_information($output, $args){
		if ($this->failed) return;
		
		$session = $this->input->post('session');
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `current_session` = ?", array($session));
		
		$data = $query->row();
		$data->loyalty_points = $this->get_customer_points($data->id);
		if ($output == 'json') echo json_encode($data);
	}
	
	function Get_customer_points($id){
		$query = $this->db->query("SELECT SUM(FLOOR(`amount`)) AS `total_amount` FROM `sales` WHERE `customer_id` = ? AND `is_redemption` <> 1 AND `type` = 'history'", array($id));
	    $point_sales = $query->row();
	    
	    $query = $this->db->query("SELECT SUM(FLOOR(`total_delivered`)) AS `total_amount` FROM `invoices` WHERE `contact_id` = ? AND `payment_status` = 'Paid'", array($id));
	    $point_invoices = $query->row();
	    
	    $total = $point_sales->total_amount + $point_invoices->total_amount;
	    
	    $amount = $this->system->config('member_minimum_amount');
	    $ppa = $this->system->config('member_minimum_point');
	    
	    $query = $this->db->query("SELECT SUM(`points`) AS `total_points` FROM `sale_items` WHERE `redeemed` = 1 AND `member_id` = ?", array($id));
	    $points_redeemed = $query->row();

	    return (floor($total / $amount) * $ppa) - $points_redeemed->total_points;
	}
	
	function Get_customer_history($output, $args){
		if ($this->failed) return;
		
		$id = $this->input->post('id');
		$query = $this->db->query("SELECT * FROM `sales_order` WHERE `contact_id` = ? AND `status` = 'Completed' ORDER BY `date` DESC", array($id));
		
		foreach ($query->result() as $so){
			$query = $this->db->query("SELECT * FROM `invoices` WHERE `sales_order_id` = ?", array($so->id));
			$invoice = $query->row();
			$so->invoice = $invoice;
			$sos[] = $so;
		}
		
		if ($output == 'json') echo json_encode($sos);
		
	}
	
	function Get_order_by_id($output, $args){
		if ($this->failed) return;
		
		$id = $this->input->post('id');
		$user_id = $this->input->post('user_id');
		
		$query = $this->db->query("SELECT * FROM `sales_order` WHERE `id` = ? AND `contact_id` = ?", array($id, $user_id));
		
		if ($query->num_rows()){
			$order = $query->row();

			$query = $this->db->query("SELECT * FROM `sales_order_items` WHERE `sales_order_id` = ?", array($id));
			$order->items = $query->result();
			
			$query = $this->db->query("SELECT * FROM `invoices` WHERE `sales_order_id` = ?", array($id));
			$order->invoice = $query->row();

		}
		
		if ($output == 'json') echo json_encode($order);
		
	}
	
	function Get_purchase_by_id($output, $args){
		if ($this->failed) return;
		
		$username = $this->input->post('username');
		$challenge = $this->input->post('challenge');
		$challenge_feedback = $this->input->post('challenge_feedback');
	
	}
	
	function Checkout($output, $args){
		if ($this->failed) return;
		

		
		$user 				= json_decode($this->input->post('user'));
		$items				= json_decode($this->input->post('items'));
		$gateway			= json_decode($this->input->post('gateway'));
		$shipping 			= json_decode($this->input->post('shipping'));
		$delivery			= json_decode($this->input->post('delivery'));
		$warehouse_id		= $this->input->post('warehouse_id');


		$query = $this->db->query("SELECT * FROM `contacts` WHERE `id` = ?", array($user->id));
		$contact = $query->row();		

		$address->address1	= $contact->address1;
		$address->address2	= $contact->address2;
		$address->zipcode	= $contact->zipcode;
		$address->city		= $contact->city;
		$address->state		= $contact->state;
		$address->country	= $contact->country;
		
		$date = date('Y-m-d H:i:s');
		$due_date = date('Y-m-d H:i:s',time() + (60 * 60 * 24 * 30));
		
		$this->db->insert('sales_order', array('txn_no'=>'12345'));
		
		$delivery_charges = $this->get_delivery_charges('return', array(
			'service'		=> $shipping->id,
			'country'		=> $delivery->country,
			'zipcode'		=> $delivery->zipcode,
			'warehouse_id'	=> $warehouse_id
		));
		
		$this->db->insert('sales_order', array('txn_no'=>'54321'));
		
		$txn_no = $this->serial_no('sales_order', $warehouse_id);

		$sql = $this->db->insert_string('sales_order',array(
			'txn_no'			=> $txn_no,
			'date'				=> $date,
			'due_date'			=> $due_date,
			'contact'			=> $contact->company_name ? $contact->company_name : $contact->contact_person,
			'contact_id'		=> $contact->id,
			'contact_address'	=> json_encode($address),
			'delivery_address'	=> json_encode($delivery),
			'terms'				=> 'Cash',
			'duration'			=> $this->input->post('duration'),
			'status'			=> 'Pending',
			'payment_status'	=> 'Unpaid',
			'payment_gateway'	=> $gateway->namespace,
			'payment_method'	=> $gateway->name,
			'shipping'			=> $shipping->name,
			'shipping_id'		=> $shipping->id,
			'adjustment'		=> $this->input->post('adjustment'),
			'adjustment_type'	=> $this->input->post('adjustment_type'),
			'handling_charges'	=> $delivery_charges->total_rate,
			'warehouse_id'		=> $warehouse_id,
			'tags'				=> 'Online Sales',
			'added_on'			=> date('Y-m-d H:i:s'),
			'added_by'			=> '@Plexcart'
		));

		$query = $this->db->query($sql);

		$so_id = $query->insert_id;
		
		$price_rules = json_decode($this->system->config('price_rules'));
				
		foreach ($items as $item){
			$qty = $item->qty;
			$item = $this->get_item_by_id('return',array('id'=>$item->id));
	
			if ($contact->rank){
				$item->price = $item->{'price'.$price_rules->{str_replace(' ','_',$contact->rank)}};
			}
				
			$sql = $this->db->insert_string('sales_order_items',array(
				'sales_order_id'	=> $so_id,
				'date'				=> $date,
				'type'				=> 'product',
				'item_sku'			=> $item->sku,
				'item_name'			=> $item->name,
				'item_description'	=> $item->description,
				'item_id'			=> $item->id,
				'qty'				=> $qty,
				'cost'				=> $item->cost * $qty,
				'price'				=> $subtotal = ($item->discount_price ? $item->discount_price : $item->price) * $qty,
				'price_per_unit'	=> ($item->discount_price ? $item->discount_price : $item->price),
				'warehouse_id'		=> $warehouse_id
			));
			
			$total += $subtotal;
			$total_cost += $item->cost;
			
			$this->db->query($sql);
			
		}
		
		$grand_total = $total + $delivery_charges->total_rate;
		
		$this->db->update('sales_order', array(
			'cost'	=> $total_cost,
			'total'	=> $grand_total
		),"`id` = '{$so_id}'");




		

		$query = $this->db->query("SELECT * FROM `sales_order` WHERE `id` = ?", array($so_id));
		$order = $query->row();
		
		$query = $this->db->query("SELECT * FROM `sales_order_items` WHERE `sales_order_id` = ?", array($so_id));
		$order->items = $query->result();

		if ($output == 'json') echo json_encode($order);


	}
	
	function Complete($output, $args){
		if ($this->failed) return;
		
		$time			= $this->input->post('time');
		$secret			= $this->input->post('secret');
		$order_id		= $this->input->post('id');
		$payment_status	= $this->input->post('payment_status');
		$status			= $this->input->post('status');
		$remarks		= $this->input->post('remarks');
		$txn_reference 	= $this->input->post('txn_reference');
		$txn_date		= $this->input->post('txn_date');
		
		$mytime = time();
			
		$this->db->update('sales_order', array(
			'status'			=> $status,
			'remarks'			=> $remarks,
			'payment_status'	=> $payment_status
		),"`id` = '{$order_id}'");
		
		if ($payment_status == 'Paid'){
			$query = $this->db->query("SELECT * FROM `sales_order` WHERE `id` = ?", array($order_id));
			$so = $query->row();
			
			if ($so->invoice_id) return;
			
			$txn_no = $this->serial_no('invoices', $so->warehouse_id);
		
			$sql = $this->db->insert_string('invoices',array(
				'date'				=> date('Y-m-d H:i:s'),
				'due_date'			=> $due_date,
				'txn_no'			=> $txn_no,
				'sales_order_id'	=> $order_id,
				'contact_person'	=> $so->contact_person,
				'contact'			=> $so->contact,
				'contact_id'		=> $so->contact_id,
				'contact_address'	=> $so->contact_address,
				'delivery_address'	=> $so->delivery_address,
				'delivery_status'	=> 'Pending',
				'shipping'			=> $so->shipping,
				'shipping_id'		=> $so->shipping_id,
				'terms'				=> 'Cash',
				'duration'			=> '',
				'payment_status'	=> 'Paid',
				'payment_date'		=> $txn_date,
				'payment_reference'	=> $txn_reference,
				'status'			=> 'Pending',
				'adjustment'		=> $so->adjustment,
				'adjustment_type'	=> $so->adjustment_type,
				'handling_charges'	=> $so->handling_charges,
				'remarks'			=> $so->remarks,
				'total'				=> $so->total,
				'tags'				=> 'Online Sales',
				'added_on'			=> date('Y-m-d H:i:s'),
				'added_by'			=> '@Plexcart',
				'warehouse_id'		=> $so->warehouse_id
			));
	
			$query = $this->db->query($sql);
	
			$invoice_id = $query->insert_id;
			

			$sql = $this->db->update_string('sales_order', array(
				'invoice_id'	=> $invoice_id
			),"`id` = '{$order_id}'");
			$this->db->query($sql);

			$query = $this->db->query("SELECT * FROM `sales_order_items` WHERE `sales_order_id` = ?", array($order_id));
			$items = $query->result();
			
			$date = date('Y-m-d H:i:s');
			
			foreach ($items as $item){
				$sql = $this->db->insert_string('invoice_items',array(
					'invoice_id'		=> $invoice_id,
					'date'				=> $date,
					'type'				=> $item->type,
					'item_name'			=> $item->item_name,
					'item_description'	=> $item->item_description,
					'item_id'			=> $item->item_id,
					'item_sku'			=> $item->item_sku,
					'service_id'		=> $item->service_id,
					'qty'				=> $item->qty,
					'price'				=> $item->price,
					'price_per_unit'	=> $item->price_per_unit,
					'discount_type'		=> $item->discount_type,
					'discount'			=> $item->discount,
					'warehouse_id'		=> $item->warehouse_id
				));
				
				$this->db->query($sql);
				
			}
			
		}
		
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `id` = ?", array($so->contact_id));
		$contact = $query->row();
		
		$total 				= number_format($so->total - $so->handling_charges,2);
		$handling_charges	= number_format($so->handling_charges,2);
		$grand_total 		= number_format($so->total,2);
		
		$styles = '<style>' . $this->system->config('online_sales_email_css') . '</style>';
		
		$order_table = "<table style=\"border-collapse: collapse;\"><thead><tr><th>No</th><th>Item</th><th>Unit Price</th><th>Qty</th><th>Subtotal</th></tr></thead>";
		$order_table .= "<tfoot>
							<tr>
								<td colspan=\"4\">Total</td>
								<td style=\"text-align: right\">{$total}</td>
							</tr>
							<tr>
								<td colspan=\"4\">Shipping: {$so->shipping}</td>
								<td style=\"text-align: right\">{$handling_charges}</td>
							</tr>
							<tr>
								<td colspan=\"4\"><strong>Grand Total</strong></td>
								<td style=\"text-align: right\">{$grand_total}</td>	
							</tr>
						</tfoot>";
		$order_table .= "<tbody>";
		
		foreach ($items as $item){
			$z++;
			$order_table .= "<tr>
								<td style=\"text-align: center\">{$z}</td>
								<td>{$item->item_name}<br />{$item->item_sku}</td>
								<td style=\"text-align: center\">{$item->price_per_unit}</td>
								<td style=\"text-align: center\">{$item->qty}</td>
								<td style=\"text-align: right\">{$item->price}</td>
							</tr>";
		}
		$order_table .= "</tbody></table>";
		
		$address = json_decode($so->contact_address);
		$billing_address .= $address->address1 ? $address->address1 . "<br />"  : '';
		$billing_address .= $address->address2 ? $address->address2 . "<br />"  : '';
		$billing_address .= $address->zipcode ? $address->zipcode . ' ' : '';
		$billing_address .= $address->city ? $address->city . "<br />" : '';
		$billing_address .= $address->state ? $address->state . "<br />" : '';
		$billing_address .= $address->country ? $address->country : ''; 
				
		$address = json_decode($so->delivery_address);
    	$delivery_address = $address->name ? $address->name . "<br />" : '';
    	$delivery_address .= $address->mobile ? 'Contact No: ' . $address->mobile . "<br />"  : '';
    	$delivery_address .= $address->address1 ? $address->address1 . "<br />"  : '';
    	$delivery_address .= $address->address2 ? $address->address2 . "<br />"  : '';
    	$delivery_address .= $address->zipcode ? $address->zipcode . ' ' : '';
    	$delivery_address .= $address->city ? $address->city . "<br />"  : '';
    	$delivery_address .= $address->state ? $address->state . "<br />"  : '';
    	$delivery_address .= $address->country ? $address->country : '';

	
		if ($so->warehouse_id){
			$query = $this->db->query("SELECT * FROM `warehouses` WHERE `id` = ?", array($so->warehouse_id));
			$warehouse = $query->row();					
		}
		
		$message = $this->__template(nl2br($this->system->config('online_sales_email_new_order')),array(
			'{name}'			=> $contact->contact_person,
			'{mobile}'			=> $contact->contact_number,
			'{email}'			=> $contact->email,
			'{order_no}'		=> $so->id,
			'{datetime}'		=> date('j:iA d/m/Y',strtotime($so->datetime)),
			'{bill_addr}'		=> $billing_address,
			'{delivery_addr}'	=> $delivery_address,
			'{gateway}'			=> $so->gateway,
			'{shipping}'		=> $so->shipping,
			'{order_table}'		=> $order_table,
			'{warehouse}'		=> $warehouse->name,
			'{status}'			=> $status,
			'{payment_status}'	=> $payment_status
		));
		
		$message = '<html><head>' . $styles . '</head><body>' . $message . '</body></html>';
		
		$subject = $this->__template($this->system->config('online_sales_email_new_order_subject'),array(
			'{name}'			=> $contact->contact_person,
			'{mobile}'			=> $contact->contact_number,
			'{email}'			=> $contact->email,
			'{order_no}'		=> $so->id,
			'{datetime}'		=> date('j:iA d/m/Y',strtotime($so->datetime)),
			'{bill_addr}'		=> $billing_address,
			'{delivery_addr}'	=> $delivery_address,
			'{gateway}'			=> $so->gateway,
			'{shipping}'		=> $so->shipping,
			'{warehouse}'		=> $warehouse->name,
			'{status}'			=> $status,
			'{payment_status}'	=> $payment_status
		));
		
		$message_bo = $this->__template(nl2br($this->system->config('online_sales_email_new_order_bo')),array(
			'{name}'			=> $contact->contact_person,
			'{mobile}'			=> $contact->contact_number,
			'{email}'			=> $contact->email,
			'{order_no}'		=> $so->id,
			'{datetime}'		=> date('j:iA d/m/Y',strtotime($so->datetime)),
			'{bill_addr}'		=> $billing_address,
			'{delivery_addr}'	=> $delivery_address,
			'{gateway}'			=> $so->gateway,
			'{shipping}'		=> $so->shipping,
			'{order_table}'		=> $order_table,
			'{warehouse}'		=> $warehouse->name,
			'{status}'			=> $status,
			'{payment_status}'	=> $payment_status
		));
		
		$message_bo = '<html><head>' . $styles . '</head><body>' . $message_bo . '</body></html>';
		
		$subject_bo = $this->__template($this->system->config('online_sales_email_new_order_bo_subject'),array(
			'{name}'			=> $contact->contact_person,
			'{mobile}'			=> $contact->contact_number,
			'{email}'			=> $contact->email,
			'{order_no}'		=> $so->id,
			'{datetime}'		=> date('j:iA d/m/Y',strtotime($so->datetime)),
			'{bill_addr}'		=> $billing_address,
			'{delivery_addr}'	=> $delivery_address,
			'{gateway}'			=> $so->gateway,
			'{shipping}'		=> $so->shipping,
			'{warehouse}'		=> $warehouse->name,
			'{status}'			=> $status,
			'{payment_status}'	=> $payment_status
		));

		
    	
    	$config['protocol'] = 'smtp';
    	$config['mailpath'] = '/usr/sbin/sendmail';
    	$config['charset'] = 'iso-8859-1';
    	$config['wordwrap'] = TRUE;
    	$config['mailtype'] = 'html';
    
    	$config['smtp_host'] = $this->system->config('online_sales_responder_hostname');
    	$config['smtp_user'] = $this->system->config('online_sales_responder_email');
    	$config['smtp_pass'] = $this->system->config('online_sales_responder_password');
    	$config['smtp_port'] = $this->system->config('online_sales_responder_port');

    	$this->load->library('email');		
    	$this->email->initialize($config);

    	$this->email->from($this->system->config('online_sales_responder_email'), $this->system->config('online_sales_responder_name'));
    	$this->email->to($contact->email);
    	$this->email->bcc($this->system->config('online_sales_responder_txn_email'));
    	$this->email->subject($subject);
    	$this->email->message($message);
    	$this->email->send();
    	
    	
    	$this->email->from($this->system->config('online_sales_responder_email'), $this->system->config('online_sales_responder_name'));
    	if ($so->warehouse_id){
	    	$this->email->to($this->system->config('online_sales_responder_email') . ',' . $warehouse->email);
    	} else {
	    	$this->email->to($this->system->config('online_sales_responder_email'));
    	}
    	$this->email->bcc(array($this->system->config('online_sales_responder_txn_email')));
    	$this->email->subject($subject_bo);
    	$this->email->message($message_bo);
    	$this->email->send();

    	$notification->type 	= 'New Order';
    	$notification->datetime	= date('Y-m-d H:i:s');
    	$notification->by		= 'Auto-Responder';
    	$notification->status	= $payment_status;
    	
    	$notifications[] = $notification;
  
  		$this->db->update('sales_order', array(
  			'notification'	=> json_encode($notifications)
  		), "`id` = '{$so->id}'");

			
		$order->order_id		= $order_id;
		$order->time			= $time;
		$order->status			= $status;
		$order->payment_status	= $payment_status;
		$order->remarks			= $remarks;
		
		if ($output == 'json') echo json_encode($order);
		
		
		
	
	}
	
	function Get_payment_gateways($output, $args){
		if ($this->failed) return;
	
		$query = $this->db->query("SELECT * FROM `payment_gateways` WHERE `status` = 'Active'");
		if ($output == 'json') echo json_encode($query->result());
	}
	
	function Get_payment_gateway_by_id($output, $args){
		if ($this->failed) return;
		
		$id = $this->input->post('id');
		$query = $this->db->query("SELECT * FROM `payment_gateways` WHERE `id` = ?", array($id));
		
		if ($output == 'json') echo json_encode($query->row());
	}
	
	function Get_shipping_services($output, $args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT * FROM `shipping`");
		
		if ($output == 'json') echo json_encode($query->result());
		
	}

	
	function Get_shipping_service_by_id($output, $args){
		if ($this->failed) return;
		
		$id = $this->input->post('id');
		$query = $this->db->query("SELECT * FROM `shipping` WHERE `id` = ?", array($id));
		
		if ($output == 'json') echo json_encode($query->row());
	}
	
	function Get_warehouses($output, $args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT * FROM `warehouses`");
		if ($output == 'json') echo json_encode($query->result());		
		
	}
	
	function Get_warehouse_by_id($output, $args){
		if ($this->failed) return;
				
		$id = $this->input->post('id');
		$query = $this->db->query("SELECT * FROM `warehouses` WHERE `id` = ?", array($id));
		if ($output == 'json') echo json_encode($query->row());	
	}
	
	function Get_delivery_charges($output, $args){
		if ($this->failed) return;
		
		if ($args){
			$zipcode = $args['zipcode'];
			$country = $args['country'];
			$service = $args['service'];
			
			$warehouse_id = $args['branch'];
		} else {
			$zipcode = $this->input->post('zipcode');
			$country = $this->input->post('country');
			$service = $this->input->post('service');
			
			$warehouse_id = $this->input->post('warehouse_id');
		}
		$items = json_decode($this->input->post('items'));
		
		$query = $this->db->query("SELECT * FROM `shipping` WHERE `id` = ?", array($service));
		$service = $query->row();
		
		
		if ($country == $this->system->config('country', $warehouse_id)){
			// local zone
			$query = $this->db->query("SELECT * FROM `shipping_local_zones` WHERE `zipcode_from` <= ? AND `zipcode_to` >= ?", array($zipcode, $zipcode));
			$zone = $query->row();
			
			$query = $this->db->query("SELECT * FROM `shipping_rates` WHERE `service_id` = ? AND `zone_id` = ? AND `zone_type` = 'Local'", array($service->id, $zone->id));
			$rate = $query->row();
						
		} else {

			$query = $this->db->query("SELECT * FROM `shipping_international_zones` WHERE FIND_IN_SET(?, `countries`)", array($country));
			$zone = $query->row();
			
			$query = $this->db->query("SELECT * FROM `shipping_rates` WHERE `service_id` = ? AND `zone_id` = ? AND `zone_type` = 'International'", array($service->id, $zone->id));
			$rate = $query->row();

		}

		foreach ($items as $item){

			$query = $this->db->query("SELECT * FROM `inventory` WHERE `id` = ?", array($item->id));
			$xitem = $query->row();
			

			if ($service->measurements_type == 'Metric'){
				
				$item_weight = $this->__convertWeight($xitem->weight, $xitem->weight_unit, 'kg');
				$item_volume = $this->__convertVolume($xitem->volume, $xitem->volume_unit, 'cm3');
				$item_volumetric_weight = $item_volume / $service->shipping_factor;
				

				$weight = $item_weight >= $item_volumetric_weight ? $item_weight : $item_volumetric_weight;
				$weight *= $item->qty;
				
			} else {
				
				$item_weight = $this->__convertWeight($xitem->weight, $xitem->weight_unit, 'lb');
				$item_volume = $this->__convertVolume($xitem->volume, $xitem->volume_unit, 'in3');
				$item_volumetric_weight = $item_volume / $service->shipping_factor;
				
				$weight = $item_weight >= $item_volumetric_weight ? $item_weight : $item_volumetric_weight;
				$weight *= $item->qty;
				
			}
			
			$total_items += $item->qty;
			$total_weight += $weight;		
			

			
			if ($weight <= $rate->first_weight){
			    $shipping_rate = $rate->flat_rate + $rate->first_weight_rate;
			} elseif ($weight >= $rate->max_weight_rate){
			    $shipping_rate = $rate->flat_rate + $rate->max_weight_rate;
			} else {
				$subsequent_rate = ceil(($weight - $rate->first_weight) / $rate->subsequent_weight) * $rate->subsequent_weight_rate;
			    $shipping_rate = $rate->flat_rate + $rate->first_weight_rate + $subsequent_rate;
			}
			

			$total_rates += $shipping_rate;

		}

		$data->total_rate = $total_rates;
		$data->total_weight = $total_weight;
		$data->total_items = $total_items;
		$data->weight_unit = $service->measurements_type == 'Metric' ? 'kg' : 'lb';
		
		if ($output == 'json') echo json_encode($data);
		else return $data;
		
	}

	function Category_map($output, $args){
		if ($this->failed) return;
		
		$query = $this->db->query("SELECT DISTINCT `category`, `subcategory` FROM `inventory` WHERE `warehouse_id` = 1  ORDER BY `category` ASC, `subcategory` ASC");
		foreach ($query->result() as $map){
			$category[$map->category][] = $map->subcategory;
		}
		
		$data->map = $category;
		
		if ($output == 'json') echo json_encode($data);
		else return $data;

	}

	
	function Get_item_quantity($output, $args){
		if ($this->failed) return;
		
		$warehouse = $args['warehouse'] ? $args['warehouse'] : '';
		$item_id = $args['item_id'] ? $args['item_id'] : '';
		
		if ($warehouse == '*' || $warehouse == ''){
			$query = $this->db->query("SELECT SUM(`received`) AS `total_inbound` FROM `purchase_order_items` WHERE `item_id` = ?  GROUP BY `item_id`", array($item_id));
			$inbound = $query->row();			
			
			$query = $this->db->query("SELECT SUM(`received`) AS `total_outbound` FROM `debit_note_items` WHERE `item_id` = ? GROUP BY `item_id`", array($item_id));
			$returned = $query->row();
			
			$query = $this->db->query("SELECT SUM(`received`) AS `total_inbound` FROM `credit_note_items` WHERE `item_id` = ? GROUP BY `item_id`", array($item_id));
			$c_returned = $query->row();
			
			$query = $this->db->query("SELECT SUM(`delivered`) AS `total_outbound`, SUM(`qty`) AS `total_to_outbound` FROM `invoice_items` WHERE `item_id` = ? AND `no_do` IS NULL GROUP BY `item_id`", array($item_id));
			$invoiced = $query->row();
			
			$query = $this->db->query("SELECT SUM(`qty`) AS `total_outbound` FROM `sale_items` WHERE `item_id` = ? AND `sold` = 1  GROUP BY `item_id`", array($item_id));
			$outbound = $query->row();

		} else {
			$query = $this->db->query("SELECT SUM(`qty`) AS `total_inbound` FROM `warehouse_movement_items` WHERE `item_id` = ? AND `to_warehouse` = ? GROUP BY `item_id`", array($item_id, $warehouse));
			$inbound_movement = $query->row();
			
			$query = $this->db->query("SELECT SUM(`received`) AS `total_inbound` FROM `purchase_order_items` WHERE `item_id` = ? AND `warehouse_id` = ? GROUP BY `item_id`", array($item_id, $warehouse));
			$inbound_po = $query->row();			
			
			$inbound->total_inbound = $inbound_movement->total_inbound + $inbound_po->total_inbound;
			
			$query = $this->db->query("SELECT SUM(`received`) AS `total_outbound` FROM `debit_note_items` WHERE `item_id` = ? AND `warehouse_id` = ? GROUP BY `item_id`", array($item_id, $warehouse));
			$returned = $query->row();
			
			$query = $this->db->query("SELECT SUM(`received`) AS `total_inbound` FROM `credit_note_items` WHERE `item_id` = ? AND `warehouse_id` = ? GROUP BY `item_id`", array($item_id, $warehouse));
			$c_returned = $query->row();
			
			$query = $this->db->query("SELECT SUM(`delivered`) AS `total_outbound`, SUM(`qty`) AS `total_to_outbound` FROM `invoice_items` WHERE `item_id` = ? AND `warehouse_id` = ? AND `no_do` IS NULL GROUP BY `item_id`", array($item_id, $warehouse));
			$invoiced = $query->row();
			
			$query = $this->db->query("SELECT SUM(`qty`) AS `total_outbound` FROM `sale_items` WHERE `item_id` = ? AND `warehouse_id` = ? AND `sold` = 1  GROUP BY `item_id`", array($item_id, $warehouse));
			$outbound = $query->row();
		}
		

		$qty = $inbound->total_inbound - $outbound->total_outbound - $invoiced->total_to_outbound - $returned->total_outbound + $c_returned->total_inbound;
		
		$data->qty = $qty;
		
		if ($output == 'json') echo json_encode($data);
		else return $qty;
	}

	function Register_fb_user($output, $args){
		if ($this->failed) return;
		
		$userdata = json_decode($this->input->post('userdata'));
		
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `facebook_id` = ?", array($userdata->id));
		if ($query->num_rows()){

			$session = $username . time();
			$this->db->query("UPDATE `contacts` SET `current_session` = ? WHERE `facebook_id` = ?", array($session, $userdata->id));
			
			$data->data = $session;	

			if ($output == 'json') echo json_encode($data);

		} else {
			$this->db->insert('contacts', array(
				'is_customer'		=> 1,
				'contact_person'	=> $userdata->first_name . ' ' . $userdata->last_name,
				'contact_number'	=> $this->input->post('mobile'),
				'address1'			=> $this->input->post('address1'),
				'address2'			=> $this->input->post('address2'),
				'city'				=> $this->input->post('city'),
				'state'				=> $this->input->post('state'),
				'zipcode'			=> $this->input->post('zipcode'),
				'country'			=> $this->input->post('country'),
				'email'				=> $this->input->post('email'),
				'password'			=> $this->input->post('password'),
				'added_on'			=> date('Y-m-d H:i:s'),
				'added_by'			=> '@Plexcart',
				'tags'				=> 'Online Customer,Facebook User,Unverified'
			));
			
			$data->data = 'ok';
			if ($output == 'json') echo json_encode($data);
		}
		
		$username = $this->input->post('username');
		$password = $this->input->post('password');

		$query = $this->db->query("SELECT * FROM `contacts` WHERE `email` = ?", array($username));
		$customer = $query->row();
		
		if ($password == $customer->password){
		
			$session = $username . time();
			$this->db->query("UPDATE `contacts` SET `current_session` = ? WHERE `email` = ?", array($session, $username));
			
			$data->data = $session;	
		
			if ($output == 'json') echo json_encode($data);
		} else {
		
			$data->data = 'incorrect';	
			if ($output == 'json') echo json_encode($data);
			
		}
		

		if ($output == 'json') echo json_encode($data);
		else return $data;
		
	}
	
	function Uses_fb_login($output, $args){
		if ($this->failed) return;
		
		$data->enabled = $this->system->config('online_sales_fb_login');
		$data->fb_app_id = $this->system->config('online_sales_fb_app_id');
		$data->fb_app_secret = $this->system->config('online_sales_fb_app_secret');

		if ($output == 'json') echo json_encode($data);
		else return $data;
		
	}
	
	function Price_rules($output, $args){
		if ($this->failed) return;
		
		$data->rules = json_encode($this->system->config('price_rules'));
		
		if ($output == 'json') echo json_encode($data);
		else return $data;		
	}
	
	function Member_tnc($output, $args){
		if ($this->failed) return;	
		
		$data->tnc = $this->system->config('member_tnc');

		if ($output == 'json') echo json_encode($data);
		else return $data;
	}
	
	function Request_password_reset($output, $args){
		if ($this->failed) return;
		
		$email	= $this->input->post('email');
		$mobile = $this->input->post('mobile');
		
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `email` = ? AND `contact_number` = ?", array($email, $mobile));
		
		if ($query->num_rows()){
		
			$user = $query->row();

			$config['protocol'] = 'smtp';
	    	$config['mailpath'] = '/usr/sbin/sendmail';
	    	$config['charset'] = 'iso-8859-1';
	    	$config['wordwrap'] = TRUE;
	    	$config['mailtype'] = 'html';
	    
	    	$config['smtp_host'] = $this->system->config('online_sales_responder_hostname');
	    	$config['smtp_user'] = $this->system->config('online_sales_responder_email');
	    	$config['smtp_pass'] = $this->system->config('online_sales_responder_password');
	    	$config['smtp_port'] = $this->system->config('online_sales_responder_port');
	
	    	$this->load->library('email');		
	    	$this->email->initialize($config);
	    	
	    	$temp_pass = mt_rand(100,999) . mt_rand(10,99) . mt_rand(100,999);;
	    	
	    	$this->db->update('contacts', array(
	    		'password'		=> md5($temp_pass)
	    	), "`id` = '{$user->id}'");
	    	
	    	$message = $this->__template(nl2br($this->system->config('online_sales_email_password_reset')), array(
	    		'{name}'		=> $user->contact_person,
	    		'{email}'		=> $user->email,
	    		'{temp_pass}'	=> $temp_pass
	    	));
	
	    	$this->email->from($this->system->config('online_sales_responder_email'), $this->system->config('online_sales_responder_name'));
	    	$this->email->to($user->email);
	    	$this->email->bcc($this->system->config('online_sales_responder_txn_email'));
	    	$this->email->subject($this->system->config('online_sales_email_password_reset_subject'));
	    	$this->email->message($message);
	    	$this->email->send();
	    	
	    	$data->data = 'ok';
    	
    	} else {
	    	
	    	$data->data = 'failed';
	    	
    	}
		
		
		if ($output == 'json') echo json_encode($data);
		else return $data;
		
	}
	
	function Reset_password(){
		
	}
	
	function __convertWeight($value, $from, $to){
		$unit_ratios = array(
			'g'		=> 1,
			'kg'	=> 0.001,
			'oz'	=> 0.035273962,
			'lb'	=> 0.0022046226
		);
		
		return ($value / $unit_ratios[$from]) * $unit_ratios[$to];
	}
	
	function __convertVolume($value, $from, $to){
		$unit_ratios = array(
			'cm3'	=> 1,
			'l'		=> 0.001,
			'in3'	=> 0.0610237441
		);
		return ($value / $unit_ratios[$from]) * $unit_ratios[$to];
	}
	
	function __template($template, $data){
	
		foreach ($data as $key => $value){
			$template = str_replace($key, $value, $template);
		}
	
		return $template;
	
	}
	
	function Serial_no($type, $warehouse_id){
		$warehouse = $warehouse_id ? $warehouse_id : $this->user->data('branch');
		$query = $this->db->query("SELECT `txn_no` FROM `$type` WHERE `warehouse_id` = ? ORDER BY `txn_no` DESC LIMIT 1", array($warehouse));
		$row = $query->row();
		
		list($w, $txn_no) = explode('-', $row->txn_no);
		
		if (!$txn_no){
			$next_id = $warehouse . '-' . 10001;
		} else {
			$next_id = $warehouse . '-' . ($txn_no + 1);
		}

		return $next_id;
	}
	
	function Get_product_reviews($output){
	
		if ($this->failed) return;
		
	
		$id = $this->input->post('item_id');
		

		
		if (!$limit) $limit = 5;
		$page = $this->input->post('page') ? $this->input->post('page') : 1;
		$offset_limit = ($page - 1) * $limit;
		
		$query = $this->db->query("SELECT SUM(`rating`) AS `total_rating`, COUNT(`id`) AS `total_rated` FROM `product_reviews` WHERE `item_id` = ?", array($id));		
		$rating = $query->row();
		
		$result->average_rating = number_format($rating->total_rating / $rating->total_rated, 2);
		
		

		$query = $this->db->query("SELECT * FROM `product_reviews` WHERE `item_id` = ? ORDER BY `datetime` DESC LIMIT $offset_limit, $limit", array($id));
		$reviews = $query->result();
		
		$result->reviews = $reviews;
		
		
		if ($output == 'json'){
			echo json_encode($result);
		}
		
	}
	
	function Submit_product_reviews(){
		$username = $this->input->post('username');
		
		$query = $this->db->query("SELECT * FROM `contacts` WHERE `email` = ?", array($username));
		$customer = $query->row();
		
		$query = $this->db->insert('product_reviews', array(
			'item_id'		=> $this->input->post('item_id'),
			'customer_id'	=> $customer->id,
			'customer'		=> $customer->contact_person,
			'review'		=> $this->input->post('review'),
			'rating'		=> $this->input->post('rating'),
			'datetime'		=> date('Y-m-d H:i:s')
		));
	}

}