<?php
	
define("CR", chr(0x0D));
define("LF", chr(0x0A));
define("SP", chr(0x20));
define("HT", chr(0x09));
define("ESC", chr(0x1B));
define("GS", chr(0x1D));
define("FS", chr(0x1C));


function center_align($words, $length, $padding){

	$length = $length - ($padding * 2);
	$wrapped = wordwrap($words, $length);
	


	$lines = explode("\n",$wrapped);
	foreach ($lines as $line){
		$line = trim($line);
		$b = $length - strlen($line);
		$w = floor($b / 2);
		$l[] = rstr(' ', $padding) .  rstr(' ', $w) . $line;
	}

	return implode("\n", $l);
	
}

function right_align($words, $length, $padding = 0){
	$length = $length - ($padding * 2);
	$wrapped = wordwrap($words, $length);

	$lines = explode("\n",$wrapped);
	foreach ($lines as $line){
		$line = trim($line);
		$b = $length - strlen($line);
		$l[] = rstr(' ', $padding) .  rstr(' ', $b) . $line;
	}

	return implode("\n", $l);
	
}

function left_align($words, $length, $padding = 0, $shift = 0){
	$length = $length - ($padding * 2);
	$wrapped = wordwrap($words, $length);

	$lines = explode("\n",$wrapped);

	for ($i = 0; $i < count($lines); $i++){
		$line = trim($lines[$i]);
		$b = $length - strlen($line);
		if ($i != 0){
			$l[] = rstr(' ', $shift) . $line . rstr(' ', $padding) .  rstr(' ', $b);						
		} else {
			$l[] = $line . rstr(' ', $padding) .  rstr(' ', $b);			
		}

	}
	return implode("\n", $l);
	
}

function columns($data, $format){

	foreach ($data as $row){
		$line = null;
		$shift = 0;
		
		for ($i = 0; $i < count($format); $i++){

			switch (strtolower($format[$i]['align'])){
				case "right":
					$line .= right_align($row[$i], $format[$i]['length'], 0, $shift);
					break;
				case "center":
					$line .= center_align($row[$i], $format[$i]['length'], 0, $shift);
					break;
				default:
					$line .= left_align($row[$i], $format[$i]['length'], 0, $shift);
					break;
			}
			
			$shift += $format[$i]['length'];
			
		}
		
		$l[] = $line;
	}
	
	return implode("\n", $l);
	
}


function rstr($char, $n){
	for ($i = 0; $i < $n; $i++){
		$str .= $char;
	}
	return $str;
}

function em_start(){
	return ESC . chr(0x45)	. chr(0xFF);
}

function em_stop(){
	return ESC . chr(0x45)	. chr(0x00);	
}

function small_start(){
	return ESC . chr(0x4D) . chr(0x01);
}

function small_stop(){
	return ESC . chr(0x4D) . chr(0x00);
}

function double_width_start(){
	return ESC . chr(0x21) . chr(0x20);;
}

function double_width_stop(){
	return ESC . chr(0x21) . chr(0x00);
}

function double_height_start(){
	return ESC . chr(0x21) . chr(0x10);;	
}

function double_height_stop(){
	return ESC . chr(0x21) . chr(0x00);
}


function ul_start(){
	return ESC . chr(0x2D)	. chr(0xFF);
}

function ul_stop(){
	return ESC . chr(0x2D)	. chr(0x00);	
}

function page_mode_on(){
	// return ESC . chr(0x21) . chr(0x00);
}

function page_mode_off(){
	// return ESC . chr(0x4E) . chr(0xFF);
}

function beep(){
	// $x = chr(0x1B) . chr(0x70) . chr(0x01) . chr(0x0A) . chr(0x0A);
	// echo $x;
	// return $x;
	// return chr(0x1E);
	// return ESC . chr(0x28) . chr(0x41) . chr(0x04) . chr(0x00) . chr(0x30) . chr(0x01) . chr(0x03) . chr(0x0A);
	$x = chr(0x1D) . chr(0x28) . chr(0x41) . chr(0x05) . chr(0x00) . chr(0x61) . chr(0x64);
	return $x;
	return ESC . chr(0x6F);
}

function process_receipt($data, $width, $feed){
	$sd = json_decode($rawdata);
		
	$res = fopen('LAST_PRT.log','w+');
	fwrite($res, $rawdata);
	fclose($res);


	$lines .= double_width_start() . em_start();
	$lines .= center_align($sd->company_data->name, floor($width / 2)) . "\n";
	$lines .= em_stop() . double_width_stop();
	$lines .= center_align($sd->company_data->address1 . ', ' . $sd->company_data->address2, $width) . "\n";
	$lines .= center_align($sd->company_data->zipcode . ' ' . $sd->company_data->city . ', ' . $sd->company_data->state, $width) . "\n\n";

	foreach ($sd->items as $item){
		$i_array[] = array($item->item_name, $item->qty, $item->amount);
		if ($item->item_sku){
			$i_array[] = array($item->item_sku,'','');
		}
		$xsubtotal += $item->amount;
	}

	$lines .= rstr('*',$width) . "\n";
	$lines .= "DATE: " . $sd->date . "\n";
	$lines .= "RCPT NO: " . $sd->txn_no . "\n";
	$lines .= "CASHIER: " . $sd->sale->username . "\n";
			
	$lines .= rstr('*',$width) . "\n";
	
	$bal_n = $width - 12;

	$lines .= columns($i_array, array(
			  	0 => array("align"=>"left","length"=>$bal_n),
			  	1 => array("align"=>"center","length"=>6),
			  	2 => array("align"=>"right","length"=>6)
			  )) . "\n";
	$lines .= rstr('*',$width) . "\n";
	
	$subtotal = $sd->sale->amount - $sd->sale->adjustment;

	$lines .= right_align("SUBTOTAL: " . number_format($xsubtotal,2), $width, 0) . "\n";
	
	if ($sd->sale->discount != 0){
		if ($sd->sale->discount_type == 'p'){
			$disc_amount = number_format(($sd->sale->discount * $xsubtotal) / 100,2);
			$perc = $sd->sale->discount;
			$lines .= right_align("DISCOUNT {$perc}%: " . $disc_amount, $width, 0) . "\n";
		}
		if ($sd->sale->discount_type == 'd'){
			$disc_amount = number_format($sd->sale->discount,2);
			$lines .= right_align("DISCOUNT: " . $disc_amount, $width, 0) . "\n";
		}
	}
	
	if ($sd->sale->adjustment != 0){
		$lines .= right_align("ROUNDING ADJUST: "  .  number_format($sd->sale->adjustment,2), $width, 0) . "\n";			
	}
	

	$lines .= double_height_start();
	$lines .= right_align("TOTAL: " . $sd->currency . number_format($sd->sale->amount,2), $width, 0) . "\n";
	$lines .= double_height_stop();

	$lines .= right_align("TENDERED: " .   number_format($sd->sale->amount_tendered,2), $width, 0) . "\n";
	$lines .= right_align("CHANGE: " .  number_format($sd->sale->amount_change,2), $width, 0) . "\n";
	$lines .= "\n";
	

	$lines .= center_align("* * THANK YOU * *", $width) . "\n";
	$lines .= center_align("PLEASE COME AGAIN", $width) . "\n";
	

	if ($print){
		$res = fopen($port,'w');
		fputs($res, $lines);
		fputs($res, "\n\n\n\n\n\n\n\n");
		fputs($res, GS . chr(0x56) . chr(0x00)); // PARTIAL CUT
		fputs($res, ESC .chr(0x48) . chr(0x01));
		fclose($res);
	}

}


function process_chit($data, $width, $feed){
	
	if (!$width) $width = 40;
	if (!$feed) $feed = 8;
	
	$lines .= "\n";
	
	$lines .= double_height_start() . double_width_start();
	$lines .= center_align('Table ' . $data['table_no'] . ($data['reprint'] ? ' - REPRINT' : ''), $width/2)  . "\n";
	$lines .= double_width_stop();
	$lines .= left_align("Ordered by: " . $data['employee'] , $width) . "\n";
	$lines .= left_align("Date/Time at " . $data['datetime'], $width)  . "\n";
	$lines .= left_align("No of Pax: " . $data['pax_no'],$width)  . "\n";
	
	$lines .= rstr('-',$width) . "\n";
		
	foreach ($data['items'] as $item){
		$item->option = str_replace('['.$keymap[$i].']', '', $item->option);						

		$row[0]	= $item->qty . ' x';
		$row[1] = $item->name . ' ' . ($item->instant_sale ? ' (Takeaway)' : '') . ($item->option ? "\n- ". str_replace(',',"\n- ",$item->option) : '') ;
		
		$rows[] = $row;
	}
	
	
	$bal_n = ($width) - 6;
	
	$lines .= columns($rows, array(
			  	0 => array("align"=>"left","length"=>6),
			  	1 => array("align"=>"left","length"=>$bal_n)
			  )) . "\n";
	$lines .= rstr('-',$width) . "\n";
	$lines .= "\n";
	$lines .= double_height_stop() . "\n";
	
	$lines .= rstr("\n", $feed);
	$lines .= GS . chr(0x56) . chr(0x00);
	
	
	return $lines;
	

}