 
(function() {

    function deviceShake(threshold) {

        // Default velocity threshold for shake to register
        this.threshold = 20;
        
        // Use date to prevent multiple shakes firing            
        this.lastTime = new Date();     
    
        // User defined threshold option
        if (typeof threshold == 'object') {
            this.threshold = threshold;
        }
    }

    // Start listening for devicemotion
    deviceShake.prototype.start = function() {

        this.lastTime = new Date();
        this.lastX = null;
        this.lastY = null;
        this.lastZ = null;
    
        if (('ondevicemotion' in window)) {
            window.addEventListener('devicemotion', this, false);
        }
    };

    // Stop listening for devicemotion
    deviceShake.prototype.stop = function() {

        if (('ondevicemotion' in window)) {
            window.removeEventListener('devicemotion', this, false);
        }
    
        this.lastTime = new Date();
        this.lastX = null;
        this.lastY = null;
        this.lastZ = null;
    };

    // Calculates if shake did occur
    deviceShake.prototype.devicemotion = function(e) {

        var current = e.accelerationIncludingGravity;

        if((this.lastX !== null) || (this.lastY !== null) || (this.lastZ !== null)) {

            var deltaX = Math.abs(this.lastX - current.x),
                deltaY = Math.abs(this.lastY - current.y),
                deltaZ = Math.abs(this.lastZ - current.z);  

            if(((deltaX > this.threshold) && (deltaY > this.threshold)) || 
            ((deltaX > this.threshold) && (deltaZ > this.threshold)) || 
            ((deltaY > this.threshold) && (deltaZ > this.threshold))) {
        
                //calculate time in milliseconds since last shake registered
                var currentTime = new Date(),
                    timeDifference = currentTime.getTime() - this.lastTime.getTime();
            
                if (timeDifference > 300) {
                    this.shakeEventDidOccur();  
                    this.lastTime = new Date();     
                }
            }
        }
    
        this.lastX = current.x;
        this.lastY = current.y;
        this.lastZ = current.z; 
    };

    // Callback method will be implemented by user
    deviceShake.prototype.shakeEventDidOccur = function() {

    };

    // Event handler
    deviceShake.prototype.handleEvent = function(e) {

        if (typeof(this[e.type]) === 'function' ) {
            return this[e.type](e);
        }
    };

    // Public function
    window.deviceShake = deviceShake;
    
})();
